/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.model;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;
import java.util.logging.Logger;

/**
 * Class which can be used to write binaries
 *
 * @author Gerben Feenstra
 */
public class BinaryWriter extends Writer {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.isobus.model");
    private final DataOutputStream dataOutputStream;

    public BinaryWriter(OutputStream outputStream) {
        dataOutputStream = new DataOutputStream(outputStream);
    }

    /**
     * writes last byte of a long value to the wrapped data output stream.
     * Only least significant 8 bits are used, rest of the precision of the long
     * and the sign will be lost.
     *
     * ONLY SUPPORTS BIT8_UNSIGNED and BIT32_SIGNED AS OF YET!
     *
     * @param field
     * @param valueToWrite Value to write to this field
     */
    public void write(XmlField field, long valueToWrite) throws IOException {
        dataOutputStream.write(convert(valueToWrite, field.getDataType()));
    }
    
    public static byte[] convert(long value, BinaryDataType datatype) {
        switch (datatype) {
            case BIT8_UNSIGNED:
                assert value >= 0 : "Value to write cannot be < 0";
                assert value <= 255 : "Value should be smaller than 255";
                return new byte[]{(byte)value};
            case BIT32_SIGNED:
                assert value >= Integer.MIN_VALUE;
                assert value <= Integer.MAX_VALUE;
                
                // little Endian
                return new byte[]{
                    (byte) value,
                    (byte)(value >> 8),
                    (byte)(value >> 16),
                    (byte)(value >> 24),
                };
            default:
                throw new IllegalArgumentException("data type " + datatype + " is not supported yet");
        }
    }

    @Override
    public void write(char[] cbuf, int off, int len) throws IOException {
        throw new UnsupportedOperationException("writing of character arrays not supported");
    }

    @Override
    public void flush() throws IOException {
        dataOutputStream.flush();
    }

    @Override
    public void close() throws IOException {
        dataOutputStream.close();
    }
}
