/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.model;

import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;

/**
 * Manager to which handles isobus taskdata files
 * To register your implementation of this manager add ServiceProvider(Service=IsobusManager.class)
 * to your implementation
 * 
 * @author Merijn Zengers
 */
public interface IsobusManager {
    
   
    /**
     * Get all the taskdata folders from the provided project
     * These folders contain a TASKDATA.XML and optional GRD.BIN ,TLG.BIN and TLG.XML files 
     * @param project the project to search in
     * @return the found folder
     */
    public FileObject[] getTaskDataFolders(Project project);
    
    /**
     * Returns the folder where the Isobus files are stored
     * @param project the project where the isobus files belong
     * @return fileObject Isobus folder
     */
    public FileObject getIsobusFolder(Project project);
    
    /**
     * Export the given tasks to an isobus taskfile in the specified destination
     * folder.
     * 
     * 
     * @param destFolder destination folder
     * @param gridType 
     * @param ddi isobus data dictionary id
     * @param multiplier conversion factor to use when the TZN values use a different UOM than the DDI.
     * @param tasks Tasks to export
     */
    //public void exportTaskdata(FileObject destFolder, GridTypeEnum gridType, DDI ddi, double multiplier, Callback<Object> callback, ExportableTask... tasks);
}
