/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.model.util;

import java.math.BigDecimal;
import java.math.MathContext;

/**
 *
 * @author johan
 */
public class DatatypeConverters {
    
    public static final int GRID_CELL_SIZE_DECIMALS = 8;
    public static final int GRID_POSITION_DECIMALS = 8;
    
    public static String printGridCellSize(double d) {
        // round after converting to bigdecimal; a rounded double might add extra decimals during conversion since it's not precise.
        BigDecimal bd = limitDecimalPlaces(new BigDecimal(d), GRID_CELL_SIZE_DECIMALS);
        // write to xml without exponents:
        return bd.toPlainString();
    }
    
    public static String printGridPosition(BigDecimal d) {
        BigDecimal bd = limitDecimalPlaces(d, GRID_POSITION_DECIMALS);
        // write to xml without exponents:
        return bd.toPlainString();
    }
    
    protected static double limitDecimalPlaces(double value, int numberOfDecimals) {
        final double pow10 = Math.pow(10, numberOfDecimals);
        final double shiftLeft = value * pow10;
        assert shiftLeft <= Long.MAX_VALUE && shiftLeft >= Long.MIN_VALUE;
        return Math.round(shiftLeft) / pow10;
    }

    protected static BigDecimal limitDecimalPlaces(BigDecimal value, int numberOfDecimals) {
        assert numberOfDecimals >= 0;
        if (value.scale() > numberOfDecimals) {
            final int precision = value.precision() - (value.scale() - numberOfDecimals);
            assert precision > 0;
            final MathContext mc = new MathContext(precision);
            return value.round(mc);
        }
        return value;
    }
}
