/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.model.util;

import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.field.model.TreatmentZone;
import nl.cloudfarming.client.isobus.model.*;

/**
 * Can create a grid
 *
 * @author Merijn Zengers
 */
public class IsobusObjectFactoryUtil {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.isobus.model.util");

    /**
     * Create an IsobusTreatmentZone using an Agrosense TreatmentZone and a
     * TreatmentzoneCode for the task in the context of this treatmentzone. Will
     * add a ProcessDataVariable to the created IsobusTreatmentZone Will set the
     * processDataDDI to hex 8D as provided in ISO_11783-11
     *
     * @param presentation may be null
     * @param treatmentZone Agrosense Treatmentzone
     * @param treatmentZoneCode Code
     * @return Isobus Treatmentzone
     */
    public static IsobusTreatmentZone createIsobusTreatmentZone(ValuePresentation presentation, TreatmentZone treatmentZone, short treatmentZoneCode, DDI ddi, double multiplier) {
        if (treatmentZone.getValue() != null) {
            IsobusTreatmentZone isobusTreatmentZone = new IsobusTreatmentZone();
            isobusTreatmentZone.setTreatmentZoneDesignator(treatmentZone.getName());
            isobusTreatmentZone.setTreatmentZoneCode(treatmentZoneCode);
            isobusTreatmentZone.setTreatmentZoneColor(new Double(treatmentZone.getValue()).shortValue());

            // ProcessDataVariable for this treatmentzone
            ProcessDataVariable processDataVariable = createProcessDataVariable(treatmentZone.getValue(), presentation, ddi, multiplier);
            isobusTreatmentZone.getPDV().add(processDataVariable);
            return isobusTreatmentZone;
        } else {
            LOGGER.log(Level.SEVERE, "Treatment zone has no value set cannot create isobus treatment zone from it {0}", treatmentZone.getName());
        }
        return null;
    }

    /**
     * Create a {@link ProcessDataVariable} with the value and {@link ValuePresentation}
     * passed
     *
     * @param value the value to set
     * @param presentation the valuePresentation to set
     * @param ddi isobus data dictionary id
     * @return the created ProcesDataVariable
     */
    public static ProcessDataVariable createProcessDataVariable(double value, ValuePresentation presentation, DDI ddi, double multiplier) {
        ProcessDataVariable processDataVariable = new ProcessDataVariable();

        processDataVariable.setProcessDataDDI(ddi.getDDEntityId()); // A
        processDataVariable.setProcessDataValue(ddi.toExportValue(value * multiplier)); // B

        if (presentation != null) {
            processDataVariable.setValuePresentationIdRef(presentation);
        }
        return processDataVariable;
    }
    
    
    
}
